// SEASHELL_READONLY

#include <stdbool.h>

// This module provides an void * Sequence ADT

struct sequence;

// sequence_create(data_dest, data_print, data_comp) returns a pointer to a new 
//   (empty) sequence, with the following function pointers:
//   * data_dest(a):   destroys the stored data a
//   * data_print(a):  prints the stored data a
//   * data_comp(a,b): compares two stored data a and b; returns false if a,
//                     and true if b should be sorted first.
// effects: allocates memory (client must call sequence_destroy)
// time:    O(1)
struct sequence *sequence_create(void (*data_dest)(void *),
                                 void (*data_print)(const void *),
                                 bool (*data_comp)(void *, void *));

// sequence_destroy(seq) frees all memory used by the sequence seq and the
//   items stored within seq.
// effects: seq and its stored values are invalidated
// time:    O(n)
void sequence_destroy(struct sequence *seq);

// sequence_length(seq) returns the number of items currently stored in the
//   sequence seq.
// time: O(1)
int sequence_length(const struct sequence *seq);

// sequence_item_at(seq, pos) returns the item in the sequence seq at position
//   pos.
// requires: 0 <= pos < sequence_length(seq)
// time:     O(1)
const void *sequence_item_at(const struct sequence *seq, int pos);

// sequence_insert_at(seq, pos, itm) inserts the item itm at position pos into
//   the sequence seq. Items at position larger or equal than pos change their
//   position within seq by 1.
// requires: 0 <= pos <= sequence_length(seq)
// effects:  mutates seq
// time:     O(n)
void sequence_insert_at(struct sequence *seq, int pos, void *val);

// sequence_remove_at(seq, pos) removes and returns the item at position pos
//   from the sequence seq. Items at position larger than pos change their
//   position within seq by 1.
// requires: 0 <= pos < sequence_length(seq)
// effects:  mutates seq
// time:     O(n)
void *sequence_remove_at(struct sequence *seq, int pos);

// sequence_print(seq) prints out the items in the sequence seq.
// effects: writes to console
// time:    O(n)
void sequence_print(const struct sequence *seq);

// sequence_sort(seq) sorts the sequence seq using the function data_comp (see
//   sequence_create).
// effects: mutates seq
// time:    O(n^2)
void sequence_sort(struct sequence *seq);