// SEASHELL_READONLY

#include <stdbool.h>

// This module calculates the solution for the n-Queens Problem.
//   (https://en.wikipedia.org/wiki/Eight_queens_puzzle)

struct board;

// board_create(dim) creates an empty chess board with dim rows and dim columns.
// requires: dim must be positive
// effects:  allocates memory (client must call board_destroy)
// time:     O(dim^2)
struct board *board_create(int dim);

// board_destroy(brd) releases all resources used by brd.
// effects: brd is invalidated
// time:    O(1)
void board_destroy(struct board *brd);

// board_queen(brd, x, y) places a queen at column x and row y, if there was
//   no queen, or removes the queen from column x and row y, if there was one.
//   If (x,y) does not contain a queen and is currently threatened, no queen
//   is placed in brd.
// effects: might mutate brd
// time:    O(dim^2)
void board_queen(struct board *brd, int x, int y);

// board_solve(brd, count) attempts to place count queens onto the board brd,
//   so that no two queens are threatening each other. Queens that are already
//   on the board remain unchanged. If such a placement was successful, brd
//   stores the solution and the function returns true; if not, brd remains 
//   unchanged and the function returns false. If a solution was found, it can
//   be retrieved using board_get.
// effects: might mutate brd
// time:    O(n!)
bool board_solve(struct board *brd, int count);

// board_debug(brd) prints the board to the console. Queens are represented by
//   the character 'Q' and empty squares by the character '_'.
// effects: writes to console
// time:    O(dim^2)
void board_debug(const struct board *brd);

// board_get(brd) returns a representation of the board, in which fields
//   containing a queen are true and empty fields are false.
// time: O(1)
const bool *board_get(const struct board *brd);